//
//  Fixperms - set and get permissions on lists of files
//  Copyright (C) 1993  Anthony Edward Hall (aehall@seattleu.edu)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// FIXPERMS.CC - main for fixperms

#include <stdio.h>
#include <unistd.h>   // for chdir
#include <dirent.h>   // for NAME_MAX
#include <string.h>
#include <malloc.h>
#include <getopt.h>
#include "getperms.h"
#include "setperms.h"
#include "fixperms.h"
#include "version.h"

void message()  // print out GNU General Public License
  {
  fprintf(stdout,"\n    Copyright (C) 1993  Anthony Edward Hall (aehall@seattleu.edu)\n");
  fprintf(stdout,"    Version: %s  Date: %s\n",VERSION,DATE);
  fprintf(stdout,"\n    This program is free software; you can redistribute it and/or modify\n");
  fprintf(stdout,"    it under the terms of the GNU General Public License as published by\n");
  fprintf(stdout,"    the Free Software Foundation; either version 2 of the License, or\n");
  fprintf(stdout,"    (at your option) any later version.\n");
  fprintf(stdout,"\n    This program is distributed in the hope that it will be useful,\n");
  fprintf(stdout,"    but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
  fprintf(stdout,"    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
  fprintf(stdout,"    GNU General Public License for more details.\n");
  fprintf(stdout,"\n    You should have received a copy of the GNU General Public License\n");
  fprintf(stdout,"    along with this program; if not, write to the Free Software\n");
  fprintf(stdout,"    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n");
  }; // end message

int usage(char *app_name)       // display command line options
  {
  fprintf(stderr,"\nUsage: %s [-vcqM] [--verbose] [--check] [--quiet]\n",app_name);            
  fprintf(stderr,"       [--message] package\n");
  fprintf(stderr,"       %s [-vqRM] [--verbose] [--quiet] [--recursive]\n",app_name);
  fprintf(stderr,"       [--message] package file_list\n");
  return 1;
  }; // end usage

int main(int argc, char *argv[]) // main
  {
  int exit=0, options=O_NONE, i, c, args=0;
  char *pname, *list[1024];
  static struct option const long_options[]=
    {
    {"recursive", no_argument, 0, 'R'},
    {"verbose", no_argument, 0, 'v'},
    {"message", no_argument, 0, 'M'},
    {"check", no_argument, 0, 'c'},
    {"quiet", no_argument, 0, 'q'},
    {0, 0, 0, 0}
    };

  if (strrchr(argv[0],'/'))
    strcpy(argv[0],(char *)(strrchr(argv[0],'/')+1)); // "basename argv[0]"
  if (chdir(PERMS_DIR))  // check for existance of PERMS_DIR
    {
    perror(PERMS_DIR);
    return 1;
    };
  while ((c=getopt_long(argc,argv,"RvMcq",long_options,(int *)0)) != EOF)
    {     // PARSE OPTIONS
    switch (c)
      {
      case 'R':    // recursive
        options+=O_RECURSIVE;
        break;
      case 'v':   // verbose
        options+=O_VERBOSE;
        break;
      case 'c':   // check only
        options+=O_CHECK_ONLY;
        break;
      case 'q':   // quiet
        options+=O_QUIET;
        break;
      case 'M':   // GNU message
        message();
        break;
      default:
        break;
      };
    };
  if (!O_IS_QUIET(options))
    fprintf(stdout,"Fixperms!  v%-20s                  Copyright (C) 1993  A.E.Hall\n",VERSION);
  for (i=1; i < argc; i++)   // take out command line options
    {
    if ((argv[i][0] != '-') && (strcmp(argv[i],"-"))) // check for options,
      {                                              // exclude "-" stdin
      list[args]=(char *)malloc(NAME_MAX+1);
      strcpy(list[args],argv[i]);
      args += 1;
      };
    };
  switch (args)
    {
    case 0:
      exit=usage(argv[0]);        // no package specified
      break;
    case 1:
      exit=setperms(list[0],options);  // set permissions on package
      break;
    default:
      exit=getperms(args,list,options);
      if (exit == ABNORMAL_EXIT)  // if something fails delete the file
        {
        pname=(char *)malloc(255);
        sprintf(pname,"%s/%s.%s",PERMS_DIR,list[0],PERMS_SUFFIX);
        remove(pname);
        };
      break;
    };
  return exit;
  }; // end main

// end fixperms.cc
