//
//  Fixperms - set and get permissions on lists of files
//  Copyright (C) 1993  Anthony Edward Hall (aehall@seattleu.edu)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// GETPERMS.CC - create permissions packages

#include <stdio.h>
#include <sys/stat.h>  // for file mode defines
#include <string.h>
#include <malloc.h>
#include <sys/dir.h>   // for directory stuff
#include <unistd.h>    // for readlink
#include "fixperms.h"
#include "getperms.h"
#include "file.h"

// get file data
int  getfile(int argc, char *argv[], int options, char *pname, FILE *pptr)
  {
  DIR *dptr=NULL;
  struct dirent *dir;
  char *Rargv[1024];
  int i,j=1, error=0;
  file fptr;

  for (i=1; i < argc; i++)
    {
    error=fptr.get_file(argv[i]);
    fptr.output(pptr,&fptr,options);
    if (O_IS_VERBOSE(options))
      fptr.output(stdout,&fptr,options);
    if (O_IS_RECURSIVE(options) && (fptr.ftype == 'd'))
      {
      dptr=opendir(fptr.fname);
      while((dir=readdir(dptr)) !=NULL)
        {
        if (strcmp(dir->d_name,"..") && strcmp(dir->d_name,"."))
          {
          Rargv[j]=(char *)malloc(255);
          sprintf(Rargv[j],"%s/%s",fptr.fname,dir->d_name);
          j += 1;
          };
        };
      closedir(dptr);
      error=getfile(j,Rargv,options,pname,pptr); // perform recursive call
      };
    };
  return error;
  };  // end getfile

// get permissions on a list of file
int getperms(int argc, char *argv[], int options)
  {
  FILE *fptr, *pptr;
  char *pname;
  int error=0;

  pname=(char *)malloc(255);
  sprintf(pname,"%s/%s.%s",PERMS_DIR,argv[0],PERMS_SUFFIX);
  if ((fptr=fopen(pname,"r")) != NULL) // does package already exists
    {
    fprintf(stderr,"\n%s: package already exists\n",argv[0]);
    return 1;
    };
  if ((pptr=fopen(pname,"w+")) == NULL) // open package for writing
    {
    perror(pname);
    return 1;
    };
  if (!O_IS_QUIET(options))
    fprintf(stdout,"\n%s: creating package\n",argv[0]);
  error=getfile(argc,argv,options,pname,pptr);
  fclose(pptr);
  if ((error == 0) && !O_IS_QUIET(options))
    fprintf(stdout,"%s: package created\n",argv[0]);
  else
    fprintf(stderr,"%s: package not created\n",argv[0]);
  return error;
  }; // end getperms

// end getperms.cc
